# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.errorreporting_v1beta1.types import common


__protobuf__ = proto.module(
    package="google.devtools.clouderrorreporting.v1beta1",
    manifest={"GetGroupRequest", "UpdateGroupRequest",},
)


class GetGroupRequest(proto.Message):
    r"""A request to return an individual group.

    Attributes:
        group_name (str):
            Required. The group resource name. Written as
            ``projects/{projectID}/groups/{group_name}``. Call
            ```groupStats.list`` <https://cloud.google.com/error-reporting/reference/rest/v1beta1/projects.groupStats/list>`__
            to return a list of groups belonging to this project.

            Example: ``projects/my-project-123/groups/my-group``
    """

    group_name = proto.Field(proto.STRING, number=1,)


class UpdateGroupRequest(proto.Message):
    r"""A request to replace the existing data for the given group.

    Attributes:
        group (google.cloud.errorreporting_v1beta1.types.ErrorGroup):
            Required. The group which replaces the
            resource on the server.
    """

    group = proto.Field(proto.MESSAGE, number=1, message=common.ErrorGroup,)


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2016 Google LLC All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility functions for Error Reporting."""

from google.cloud.error_reporting.client import HTTPContext


def build_flask_context(request):
    """Builds an HTTP context object from a Flask (Werkzeug) request object.

     This helper method extracts the relevant HTTP context from a Flask request
     object into an object ready to be sent to Error Reporting.

    .. code-block:: python

        >>> @app.errorhandler(HTTPException)
        ... def handle_error(exc):
        ...     client.report_exception(
        ...         http_context=build_flask_context(request))
        ...     # rest of error response code here

    :type request: :class:`werkzeug.wrappers.request`
    :param request: The Flask request object to convert.

    :rtype: :class:`~google.cloud.error_reporting.client.HTTPContext`
    :returns: An HTTPContext object ready to be sent to the Error Reporting
              API.
    """
    return HTTPContext(
        url=request.url,
        method=request.method,
        user_agent=request.user_agent.string,
        referrer=request.referrer,
        remote_ip=request.remote_addr,
    )



# Copyright 2017 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import mock


def _make_credentials():
    import google.auth.credentials

    return mock.Mock(spec=google.auth.credentials.Credentials)


class Test_ErrorReportingLoggingAPI(unittest.TestCase):

    PROJECT = "PROJECT"

    def _make_one(self, project, credentials, **kw):
        from google.cloud.error_reporting._logging import _ErrorReportingLoggingAPI

        return _ErrorReportingLoggingAPI(project, credentials, **kw)

    @mock.patch("google.cloud.logging.Client")
    def test_ctor_defaults(self, mocked_cls):
        credentials = _make_credentials()

        logging_api = self._make_one(self.PROJECT, credentials)

        self.assertIs(logging_api.logging_client, mocked_cls.return_value)
        mocked_cls.assert_called_once_with(
            project=self.PROJECT,
            credentials=credentials,
            _http=None,
            client_info=None,
            client_options=None,
        )

    @mock.patch("google.cloud.logging.Client")
    def test_ctor_explicit(self, mocked_cls):
        credentials = _make_credentials()
        http = mock.Mock()
        client_info = mock.Mock()
        client_options = mock.Mock()

        logging_api = self._make_one(
            self.PROJECT,
            credentials,
            _http=http,
            client_info=client_info,
            client_options=client_options,
        )

        self.assertIs(logging_api.logging_client, mocked_cls.return_value)
        mocked_cls.assert_called_once_with(
            project=self.PROJECT,
            credentials=credentials,
            _http=http,
            client_info=client_info,
            client_options=client_options,
        )

    @mock.patch("google.cloud.logging.Client")
    def test_report_error_event(self, mocked_cls):
        credentials = _make_credentials()
        logging_api = self._make_one(self.PROJECT, credentials)

        logger = mock.Mock(spec=["log_struct"])
        logging_api.logging_client.logger.return_value = logger

        # Actually make the API call.
        error_report = {"message": "The cabs are here."}
        logging_api.report_error_event(error_report)

        # Check the mocks.
        logger.log_struct.assert_called_once_with(error_report)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.errorreporting_v1beta1.types import common
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.devtools.clouderrorreporting.v1beta1",
    manifest={
        "TimedCountAlignment",
        "ErrorGroupOrder",
        "ListGroupStatsRequest",
        "ListGroupStatsResponse",
        "ErrorGroupStats",
        "TimedCount",
        "ListEventsRequest",
        "ListEventsResponse",
        "QueryTimeRange",
        "ServiceContextFilter",
        "DeleteEventsRequest",
        "DeleteEventsResponse",
    },
)


class TimedCountAlignment(proto.Enum):
    r"""Specifies how the time periods of error group counts are
    aligned.
    """
    ERROR_COUNT_ALIGNMENT_UNSPECIFIED = 0
    ALIGNMENT_EQUAL_ROUNDED = 1
    ALIGNMENT_EQUAL_AT_END = 2


class ErrorGroupOrder(proto.Enum):
    r"""A sorting order of error groups."""
    GROUP_ORDER_UNSPECIFIED = 0
    COUNT_DESC = 1
    LAST_SEEN_DESC = 2
    CREATED_DESC = 3
    AFFECTED_USERS_DESC = 4


class ListGroupStatsRequest(proto.Message):
    r"""Specifies a set of ``ErrorGroupStats`` to return.

    Attributes:
        project_name (str):
            Required. The resource name of the Google Cloud Platform
            project. Written as ``projects/{projectID}`` or
            ``projects/{projectNumber}``, where ``{projectID}`` and
            ``{projectNumber}`` can be found in the `Google Cloud
            Console <https://support.google.com/cloud/answer/6158840>`__.

            Examples: ``projects/my-project-123``, ``projects/5551234``.
        group_id (Sequence[str]):
            Optional. List all
            <code>ErrorGroupStats</code> with these IDs.
        service_filter (google.cloud.errorreporting_v1beta1.types.ServiceContextFilter):
            Optional. List only
            <code>ErrorGroupStats</code> which belong to a
            service context that matches the filter. Data
            for all service contexts is returned if this
            field is not specified.
        time_range (google.cloud.errorreporting_v1beta1.types.QueryTimeRange):
            Optional. List data for the given time range. If not set, a
            default time range is used. The field time_range_begin in
            the response will specify the beginning of this time range.
            Only ErrorGroupStats with a non-zero count in the given time
            range are returned, unless the request contains an explicit
            group_id list. If a group_id list is given, also
            ErrorGroupStats with zero occurrences are returned.
        timed_count_duration (google.protobuf.duration_pb2.Duration):
            Optional. The preferred duration for a single returned
            ``TimedCount``. If not set, no timed counts are returned.
        alignment (google.cloud.errorreporting_v1beta1.types.TimedCountAlignment):
            Optional. The alignment of the timed counts to be returned.
            Default is ``ALIGNMENT_EQUAL_AT_END``.
        alignment_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. Time where the timed counts shall
            be aligned if rounded alignment is chosen.
            Default is 00:00 UTC.
        order (google.cloud.errorreporting_v1beta1.types.ErrorGroupOrder):
            Optional. The sort order in which the results are returned.
            Default is ``COUNT_DESC``.
        page_size (int):
            Optional. The maximum number of results to
            return per response. Default is 20.
        page_token (str):
            Optional. A ``next_page_token`` provided by a previous
            response. To view additional results, pass this token along
            with the identical query parameters as the first request.
    """

    project_name = proto.Field(proto.STRING, number=1,)
    group_id = proto.RepeatedField(proto.STRING, number=2,)
    service_filter = proto.Field(
        proto.MESSAGE, number=3, message="ServiceContextFilter",
    )
    time_range = proto.Field(proto.MESSAGE, number=5, message="QueryTimeRange",)
    timed_count_duration = proto.Field(
        proto.MESSAGE, number=6, message=duration_pb2.Duration,
    )
    alignment = proto.Field(proto.ENUM, number=7, enum="TimedCountAlignment",)
    alignment_time = proto.Field(
        proto.MESSAGE, number=8, message=timestamp_pb2.Timestamp,
    )
    order = proto.Field(proto.ENUM, number=9, enum="ErrorGroupOrder",)
    page_size = proto.Field(proto.INT32, number=11,)
    page_token = proto.Field(proto.STRING, number=12,)


class ListGroupStatsResponse(proto.Message):
    r"""Contains a set of requested error group stats.

    Attributes:
        error_group_stats (Sequence[google.cloud.errorreporting_v1beta1.types.ErrorGroupStats]):
            The error group stats which match the given
            request.
        next_page_token (str):
            If non-empty, more results are available.
            Pass this token, along with the same query
            parameters as the first request, to view the
            next page of results.
        time_range_begin (google.protobuf.timestamp_pb2.Timestamp):
            The timestamp specifies the start time to
            which the request was restricted. The start time
            is set based on the requested time range. It may
            be adjusted to a later time if a project has
            exceeded the storage quota and older data has
            been deleted.
    """

    @property
    def raw_page(self):
        return self

    error_group_stats = proto.RepeatedField(
        proto.MESSAGE, number=1, message="ErrorGroupStats",
    )
    next_page_token = proto.Field(proto.STRING, number=2,)
    time_range_begin = proto.Field(
        proto.MESSAGE, number=4, message=timestamp_pb2.Timestamp,
    )


class ErrorGroupStats(proto.Message):
    r"""Data extracted for a specific group based on certain filter
    criteria, such as a given time period and/or service filter.

    Attributes:
        group (google.cloud.errorreporting_v1beta1.types.ErrorGroup):
            Group data that is independent of the filter
            criteria.
        count (int):
            Approximate total number of events in the
            given group that match the filter criteria.
        affected_users_count (int):
            Approximate number of affected users in the given group that
            match the filter criteria. Users are distinguished by data
            in the ``ErrorContext`` of the individual error events, such
            as their login name or their remote IP address in case of
            HTTP requests. The number of affected users can be zero even
            if the number of errors is non-zero if no data was provided
            from which the affected user could be deduced. Users are
            counted based on data in the request context that was
            provided in the error report. If more users are implicitly
            affected, such as due to a crash of the whole service, this
            is not reflected here.
        timed_counts (Sequence[google.cloud.errorreporting_v1beta1.types.TimedCount]):
            Approximate number of occurrences over time.
            Timed counts returned by ListGroups are
            guaranteed to be:
            - Inside the requested time interval
            - Non-overlapping, and
            - Ordered by ascending time.
        first_seen_time (google.protobuf.timestamp_pb2.Timestamp):
            Approximate first occurrence that was ever seen for this
            group and which matches the given filter criteria, ignoring
            the time_range that was specified in the request.
        last_seen_time (google.protobuf.timestamp_pb2.Timestamp):
            Approximate last occurrence that was ever seen for this
            group and which matches the given filter criteria, ignoring
            the time_range that was specified in the request.
        affected_services (Sequence[google.cloud.errorreporting_v1beta1.types.ServiceContext]):
            Service contexts with a non-zero error count for the given
            filter criteria. This list can be truncated if multiple
            services are affected. Refer to ``num_affected_services``
            for the total count.
        num_affected_services (int):
            The total number of services with a non-zero
            error count for the given filter criteria.
        representative (google.cloud.errorreporting_v1beta1.types.ErrorEvent):
            An arbitrary event that is chosen as
            representative for the whole group. The
            representative event is intended to be used as a
            quick preview for the whole group. Events in the
            group are usually sufficiently similar to each
            other such that showing an arbitrary
            representative provides insight into the
            characteristics of the group as a whole.
    """

    group = proto.Field(proto.MESSAGE, number=1, message=common.ErrorGroup,)
    count = proto.Field(proto.INT64, number=2,)
    affected_users_count = proto.Field(proto.INT64, number=3,)
    timed_counts = proto.RepeatedField(proto.MESSAGE, number=4, message="TimedCount",)
    first_seen_time = proto.Field(
        proto.MESSAGE, number=5, message=timestamp_pb2.Timestamp,
    )
    last_seen_time = proto.Field(
        proto.MESSAGE, number=6, message=timestamp_pb2.Timestamp,
    )
    affected_services = proto.RepeatedField(
        proto.MESSAGE, number=7, message=common.ServiceContext,
    )
    num_affected_services = proto.Field(proto.INT32, number=8,)
    representative = proto.Field(proto.MESSAGE, number=9, message=common.ErrorEvent,)


class TimedCount(proto.Message):
    r"""The number of errors in a given time period.
    All numbers are approximate since the error events are sampled
    before counting them.

    Attributes:
        count (int):
            Approximate number of occurrences in the
            given time period.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Start of the time period to which ``count`` refers
            (included).
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            End of the time period to which ``count`` refers (excluded).
    """

    count = proto.Field(proto.INT64, number=1,)
    start_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
    end_time = proto.Field(proto.MESSAGE, number=3, message=timestamp_pb2.Timestamp,)


class ListEventsRequest(proto.Message):
    r"""Specifies a set of error events to return.

    Attributes:
        project_name (str):
            Required. The resource name of the Google Cloud Platform
            project. Written as ``projects/{projectID}``, where
            ``{projectID}`` is the `Google Cloud Platform project
            ID <https://support.google.com/cloud/answer/6158840>`__.

            Example: ``projects/my-project-123``.
        group_id (str):
            Required. The group for which events shall be
            returned.
        service_filter (google.cloud.errorreporting_v1beta1.types.ServiceContextFilter):
            Optional. List only ErrorGroups which belong
            to a service context that matches the filter.
            Data for all service contexts is returned if
            this field is not specified.
        time_range (google.cloud.errorreporting_v1beta1.types.QueryTimeRange):
            Optional. List only data for the given time range. If not
            set a default time range is used. The field time_range_begin
            in the response will specify the beginning of this time
            range.
        page_size (int):
            Optional. The maximum number of results to
            return per response.
        page_token (str):
            Optional. A ``next_page_token`` provided by a previous
            response.
    """

    project_name = proto.Field(proto.STRING, number=1,)
    group_id = proto.Field(proto.STRING, number=2,)
    service_filter = proto.Field(
        proto.MESSAGE, number=3, message="ServiceContextFilter",
    )
    time_range = proto.Field(proto.MESSAGE, number=4, message="QueryTimeRange",)
    page_size = proto.Field(proto.INT32, number=6,)
    page_token = proto.Field(proto.STRING, number=7,)


class ListEventsResponse(proto.Message):
    r"""Contains a set of requested error events.

    Attributes:
        error_events (Sequence[google.cloud.errorreporting_v1beta1.types.ErrorEvent]):
            The error events which match the given
            request.
        next_page_token (str):
            If non-empty, more results are available.
            Pass this token, along with the same query
            parameters as the first request, to view the
            next page of results.
        time_range_begin (google.protobuf.timestamp_pb2.Timestamp):
            The timestamp specifies the start time to
            which the request was restricted.
    """

    @property
    def raw_page(self):
        return self

    error_events = proto.RepeatedField(
        proto.MESSAGE, number=1, message=common.ErrorEvent,
    )
    next_page_token = proto.Field(proto.STRING, number=2,)
    time_range_begin = proto.Field(
        proto.MESSAGE, number=4, message=timestamp_pb2.Timestamp,
    )


class QueryTimeRange(proto.Message):
    r"""Requests might be rejected or the resulting timed count
    durations might be adjusted for lower durations.

    Attributes:
        period (google.cloud.errorreporting_v1beta1.types.QueryTimeRange.Period):
            Restricts the query to the specified time
            range.
    """

    class Period(proto.Enum):
        r"""The supported time ranges."""
        PERIOD_UNSPECIFIED = 0
        PERIOD_1_HOUR = 1
        PERIOD_6_HOURS = 2
        PERIOD_1_DAY = 3
        PERIOD_1_WEEK = 4
        PERIOD_30_DAYS = 5

    period = proto.Field(proto.ENUM, number=1, enum=Period,)


class ServiceContextFilter(proto.Message):
    r"""Specifies criteria for filtering a subset of service contexts. The
    fields in the filter correspond to the fields in ``ServiceContext``.
    Only exact, case-sensitive matches are supported. If a field is
    unset or empty, it matches arbitrary values.

    Attributes:
        service (str):
            Optional. The exact value to match against
            ```ServiceContext.service`` </error-reporting/reference/rest/v1beta1/ServiceContext#FIELDS.service>`__.
        version (str):
            Optional. The exact value to match against
            ```ServiceContext.version`` </error-reporting/reference/rest/v1beta1/ServiceContext#FIELDS.version>`__.
        resource_type (str):
            Optional. The exact value to match against
            ```ServiceContext.resource_type`` </error-reporting/reference/rest/v1beta1/ServiceContext#FIELDS.resource_type>`__.
    """

    service = proto.Field(proto.STRING, number=2,)
    version = proto.Field(proto.STRING, number=3,)
    resource_type = proto.Field(proto.STRING, number=4,)


class DeleteEventsRequest(proto.Message):
    r"""Deletes all events in the project.

    Attributes:
        project_name (str):
            Required. The resource name of the Google Cloud Platform
            project. Written as ``projects/{projectID}``, where
            ``{projectID}`` is the `Google Cloud Platform project
            ID <https://support.google.com/cloud/answer/6158840>`__.

            Example: ``projects/my-project-123``.
    """

    project_name = proto.Field(proto.STRING, number=1,)


class DeleteEventsResponse(proto.Message):
    r"""Response message for deleting error events.
    """


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Client for interacting with the Error Reporting API"""

import os
import traceback

try:
    from google.cloud.error_reporting._gapic import make_report_error_api
except ImportError:  # pragma: NO COVER
    from google.api_core import client_info  # noqa

    _HAVE_GRPC = False
else:
    from google.api_core.gapic_v1 import client_info

    _HAVE_GRPC = True

from google.cloud.client import ClientWithProject
from google.cloud.error_reporting import __version__
from google.cloud.error_reporting._logging import _ErrorReportingLoggingAPI
from google.cloud.environment_vars import DISABLE_GRPC

_DISABLE_GRPC = os.getenv(DISABLE_GRPC, False)
_USE_GRPC = _HAVE_GRPC and not _DISABLE_GRPC
_CLIENT_INFO = client_info.ClientInfo(client_library_version=__version__)


class HTTPContext(object):
    """HTTPContext defines an object that captures the parameter for the
    httpRequest part of Error Reporting API

    :type method: str
    :param method: The type of HTTP request, such as GET, POST, etc.

    :type url: str
    :param url: The URL of the request

    :type user_agent: str
    :param user_agent: The user agent information that is provided with the
                       request.

    :type referrer: str
    :param referrer: The referrer information that is provided with the
                     request.

    :type response_status_code: int
    :param response_status_code: The HTTP response status code for the request.

    :type remote_ip: str
    :param remote_ip: The IP address from which the request originated. This
                      can be IPv4, IPv6, or a token which is derived from
                      the IP address, depending on the data that has been
                      provided in the error report.
    """

    def __init__(
        self,
        method=None,
        url=None,
        user_agent=None,
        referrer=None,
        response_status_code=None,
        remote_ip=None,
    ):
        self.method = method
        self.url = url
        # intentionally camel case for mapping to JSON API expects
        # pylint: disable=invalid-name
        self.userAgent = user_agent
        self.referrer = referrer
        self.responseStatusCode = response_status_code
        self.remoteIp = remote_ip


class Client(ClientWithProject):
    """Error Reporting client. Currently Error Reporting is done by creating
    a Logging client.

    :type project: str
    :param project: the project which the client acts on behalf of. If not
                    passed falls back to the default inferred from the
                    environment.

    :type credentials: :class:`google.auth.credentials.Credentials` or
                       :class:`NoneType`
    :param credentials: The authorization credentials to attach to requests.
                        These credentials identify this application to the service.
                        If none are specified, the client will attempt to ascertain
                        the credentials from the environment.

    :type _http: :class:`~requests.Session`
    :param _http: (Optional) HTTP object to make requests. Can be any object
                  that defines ``request()`` with the same interface as
                  :meth:`requests.Session.request`. If not passed, an
                  ``_http`` object is created that is bound to the
                  ``credentials`` for the current object.
                  This parameter should be considered private, and could
                  change in the future.

    :type service: str
    :param service: An identifier of the service, such as the name of the
                    executable, job, or Google App Engine service name. This
                    field is expected to have a low number of values that are
                    relatively stable over time, as opposed to version,
                    which can be changed whenever new code is deployed.


    :type version: str
    :param version: Represents the source code version that the developer
                    provided, which could represent a version label or a Git
                    SHA-1 hash, for example. If the developer did not provide
                    a version, the value is set to default.

    :type _use_grpc: bool
    :param _use_grpc: (Optional) Explicitly specifies whether
                      to use the gRPC transport or HTTP. If unset,
                      falls back to the ``GOOGLE_CLOUD_DISABLE_GRPC``
                      environment variable.
                      This parameter should be considered private, and could
                      change in the future.

    :type client_info:
        :class:`google.api_core.client_info.ClientInfo` or
        :class:`google.api_core.gapic_v1.client_info.ClientInfo`
    :param client_info:
        The client info used to send a user-agent string along with API
        requests. If ``None``, then default info will be used. Generally,
        you only need to set this if you're developing your own library
        or partner tool.

    :type client_options: :class:`~google.api_core.client_options.ClientOptions`
        or :class:`dict`
    :param client_options: (Optional) Client options used to set user options
        on the client. API Endpoint should be set through client_options.

    :raises: :class:`ValueError` if the project is neither passed in nor
             set in the environment.
    """

    SCOPE = ("https://www.googleapis.com/auth/cloud-platform",)
    """The scopes required for authenticating as an API consumer."""

    def __init__(
        self,
        project=None,
        credentials=None,
        _http=None,
        service=None,
        version=None,
        client_info=_CLIENT_INFO,
        client_options=None,
        _use_grpc=None,
    ):
        super(Client, self).__init__(
            project=project, credentials=credentials, _http=_http
        )
        self._report_errors_api = None

        self.service = service if service else self.DEFAULT_SERVICE
        self.version = version
        self._client_info = client_info
        self._client_options = client_options

        if _use_grpc is None:
            self._use_grpc = _USE_GRPC
        else:
            self._use_grpc = _use_grpc

    DEFAULT_SERVICE = "python"

    @property
    def report_errors_api(self):
        """Helper for logging-related API calls.

        See
        https://cloud.google.com/logging/docs/reference/v2/rest/v2/entries
        https://cloud.google.com/logging/docs/reference/v2/rest/v2/projects.logs

        :rtype:
            :class:`_gapic._ErrorReportingGapicApi`
            or
            :class:`._logging._ErrorReportingLoggingAPI`
        :returns: A class that implements the report errors API.
        """
        if self._report_errors_api is None:
            if self._use_grpc:
                self._report_errors_api = make_report_error_api(self)
            else:
                self._report_errors_api = _ErrorReportingLoggingAPI(
                    self.project,
                    self._credentials,
                    self._http,
                    self._client_info,
                    self._client_options,
                )
        return self._report_errors_api

    def _build_error_report(
        self, message, report_location=None, http_context=None, user=None
    ):
        """Builds the Error Reporting object to report.

        This builds the object according to

        https://cloud.google.com/error-reporting/docs/formatting-error-messages

        :type message: str
        :param message: The stack trace that was reported or logged by the
                   service.

        :type report_location: dict
        :param report_location:  The location in the source code where the
               decision was made to report the error, usually the place
               where it was logged. For a logged exception this would be the
               source line where the exception is logged, usually close to
               the place where it was caught.

               This should be a Python dict that contains the keys 'filePath',
               'lineNumber', and 'functionName'

        :type http_context: :class`google.cloud.error_reporting.HTTPContext`
        :param http_context: The HTTP request which was processed when the
                             error was triggered.

        :type user: str
        :param user: The user who caused or was affected by the crash. This can
                     be a user ID, an email address, or an arbitrary token that
                     uniquely identifies the user. When sending an error
                     report, leave this field empty if the user was not
                     logged in. In this  case the Error Reporting system will
                     use other data, such as remote IP address,
                     to distinguish affected users.
        :rtype: dict
        :returns: A dict payload ready to be serialized to JSON and sent to
                  the API.
         """
        payload = {
            "serviceContext": {"service": self.service},
            "message": "{0}".format(message),
        }

        if self.version:
            payload["serviceContext"]["version"] = self.version

        if report_location or http_context or user:
            payload["context"] = {}

        if report_location:
            payload["context"]["reportLocation"] = report_location

        if http_context:
            http_context_dict = http_context.__dict__
            # strip out None values
            payload["context"]["httpRequest"] = {
                key: value
                for key, value in http_context_dict.items()
                if value is not None
            }
        if user:
            payload["context"]["user"] = user
        return payload

    def _send_error_report(
        self, message, report_location=None, http_context=None, user=None
    ):
        """Makes the call to the Error Reporting API.

        This is the lower-level interface to build and send the payload,
        generally users will use either report() or report_exception() to
        automatically gather the parameters for this method.

        :type message: str
        :param message: The stack trace that was reported or logged by the
                   service.

        :type report_location: dict
        :param report_location:  The location in the source code where the
               decision was made to report the error, usually the place
               where it was logged. For a logged exception this would be the
               source line where the exception is logged, usually close to
               the place where it was caught.

               This should be a Python dict that contains the keys 'filePath',
               'lineNumber', and 'functionName'

        :type http_context: :class`google.cloud.error_reporting.HTTPContext`
        :param http_context: The HTTP request which was processed when the
                             error was triggered.

        :type user: str
        :param user: The user who caused or was affected by the crash. This can
                     be a user ID, an email address, or an arbitrary token that
                     uniquely identifies the user. When sending an error
                     report, leave this field empty if the user was not
                     logged in. In this  case the Error Reporting system will
                     use other data, such as remote IP address,
                     to distinguish affected users.
        """
        error_report = self._build_error_report(
            message, report_location, http_context, user
        )
        self.report_errors_api.report_error_event(error_report)

    def report(self, message, http_context=None, user=None):
        """ Reports a message to Error Reporting

        https://cloud.google.com/error-reporting/docs/formatting-error-messages

        :type message: str
        :param message: A user-supplied message to report

        :type http_context: :class`google.cloud.error_reporting.HTTPContext`
        :param http_context: The HTTP request which was processed when the
                             error was triggered.

        :type user: str
        :param user: The user who caused or was affected by the crash. This
                     can be a user ID, an email address, or an arbitrary
                     token that uniquely identifies the user. When sending
                     an error report, leave this field empty if the user
                     was not logged in. In this case the Error Reporting
                     system will use other data, such as remote IP address,
                     to distinguish affected users.

        Example:

        .. code-block:: python

          >>>  client.report("Something went wrong!")
        """
        stack = traceback.extract_stack()
        last_call = stack[-2]
        file_path = last_call[0]
        line_number = last_call[1]
        function_name = last_call[2]
        report_location = {
            "filePath": file_path,
            "lineNumber": line_number,
            "functionName": function_name,
        }

        self._send_error_report(
            message,
            http_context=http_context,
            user=user,
            report_location=report_location,
        )

    def report_exception(self, http_context=None, user=None):
        """ Reports the details of the latest exceptions to Error Reporting.

          :type http_context: :class`google.cloud.error_reporting.HTTPContext`
          :param http_context: The HTTP request which was processed when the
                               error was triggered.

          :type user: str
          :param user: The user who caused or was affected by the crash. This
                       can be a user ID, an email address, or an arbitrary
                       token that uniquely identifies the user. When sending an
                       error report, leave this field empty if the user was
                       not logged in. In this case the Error Reporting system
                       will use other data, such as remote IP address,
                       to distinguish affected users.

           Example::

                >>>     try:
                >>>         raise NameError
                >>>     except Exception:
                >>>         client.report_exception()
        """
        self._send_error_report(
            traceback.format_exc(), http_context=http_context, user=user
        )



# -*- coding: utf-8 -*-
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import time

from google.cloud import errorreporting_v1beta1


class TestSystemReportErrorsService(object):
    def test_report_error_event(self):
        project_id = os.environ["PROJECT_ID"]

        client = errorreporting_v1beta1.ReportErrorsServiceClient()
        project_name = f"projects/{project_id}"
        message = "[MESSAGE]"
        service = "[SERVICE]"
        service_context = {"service": service}
        file_path = "path/to/file.lang"
        line_number = 42
        function_name = "meaningOfLife"
        report_location = {
            "file_path": file_path,
            "line_number": line_number,
            "function_name": function_name,
        }
        context = {"report_location": report_location}
        event = {
            "message": message,
            "service_context": service_context,
            "context": context,
        }
        response = client.report_error_event(
            request={"project_name": project_name, "event": event}
        )



# Copyright 2016 Google LLC All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Interact with Error Reporting via Logging API.

It's possible to report Error Reporting errors by formatting
structured log messages in Cloud Logging in a given format. This
client provides a mechanism to report errors using that technique.
"""

import google.cloud.logging


class _ErrorReportingLoggingAPI(object):
    """Report to Error Reporting via Logging API

    :type project: str
    :param project: the project which the client acts on behalf of. If not
                    passed falls back to the default inferred from the
                    environment.

    :type credentials: :class:`google.auth.credentials.Credentials` or
                       :class:`NoneType`
    :param credentials: The authorization credentials to attach to requests.
                        These credentials identify this application to the service.
                        If none are specified, the client will attempt to ascertain
                        the credentials from the environment.

    :type _http: :class:`~requests.Session`
    :param _http: (Optional) HTTP object to make requests. Can be any object
                  that defines ``request()`` with the same interface as
                  :meth:`requests.Session.request`. If not passed, an
                  ``_http`` object is created that is bound to the
                  ``credentials`` for the current object.
                  This parameter should be considered private, and could
                  change in the future.

    :type client_info:
        :class:`google.api_core.client_info.ClientInfo` or
        :class:`google.api_core.gapic_v1.client_info.ClientInfo`
    :param client_info:
        The client info used to send a user-agent string along with API
        requests. If ``None``, then default info will be used. Generally,
        you only need to set this if you're developing your own library
        or partner tool.

    :type client_options: :class:`~google.api_core.client_options.ClientOptions`
        or :class:`dict`
    :param client_options: (Optional) Client options used to set user options
        on the client. API Endpoint should be set through client_options.
    """

    def __init__(
        self,
        project,
        credentials=None,
        _http=None,
        client_info=None,
        client_options=None,
    ):
        self.logging_client = google.cloud.logging.Client(
            project=project,
            credentials=credentials,
            _http=_http,
            client_info=client_info,
            client_options=client_options,
        )

    def report_error_event(self, error_report):
        """Report error payload.

        :type error_report: dict
        :param: error_report:
            dict payload of the error report formatted according to
            https://cloud.google.com/error-reporting/docs/formatting-error-messages
            This object should be built using
            :meth:~`google.cloud.error_reporting.client._build_error_report`
        """
        logger = self.logging_client.logger("errors")
        logger.log_struct(error_report)



# Copyright 2017 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

import mock


class Test_make_report_error_api(unittest.TestCase):
    @staticmethod
    def _call_fut(client):
        from google.cloud.error_reporting._gapic import make_report_error_api

        return make_report_error_api(client)

    def test_make_report_error_api(self):
        client = mock.Mock(
            spec=["project", "_credentials", "_client_info", "_client_options"]
        )

        # Call the function being tested.
        patch = mock.patch(
            "google.cloud.errorreporting_v1beta1.ReportErrorsServiceClient"
        )

        with patch as patched:
            report_error_client = self._call_fut(client)

        # Assert that the final error client has the project in
        # the expected location.
        self.assertIs(report_error_client._project, client.project)
        self.assertIs(report_error_client._gapic_api, patched.return_value)
        patched.assert_called_once_with(
            credentials=client._credentials,
            client_info=client._client_info,
            client_options=client._client_options,
        )


class Test_ErrorReportingGapicApi(unittest.TestCase):

    PROJECT = "PROJECT"

    def _make_one(self, gapic_api, project):
        from google.cloud.error_reporting._gapic import _ErrorReportingGapicApi

        return _ErrorReportingGapicApi(gapic_api, project)

    def test_constructor(self):
        gapic_api = mock.Mock(spec=[])
        gapic_client_wrapper = self._make_one(gapic_api, self.PROJECT)

        self.assertEqual(gapic_client_wrapper._project, self.PROJECT)
        self.assertEqual(gapic_client_wrapper._gapic_api, gapic_api)

    def test_report_error_event(self):
        import google.cloud.errorreporting_v1beta1

        gapic_api = mock.Mock(spec=["report_error_event"])
        gapic_client_wrapper = self._make_one(gapic_api, self.PROJECT)

        error_report = {"message": "The cabs are here."}
        gapic_client_wrapper.report_error_event(error_report)

        project_name = f"projects/{self.PROJECT}"
        error_pb = google.cloud.errorreporting_v1beta1.ReportedErrorEvent(
            message=error_report["message"]
        )
        gapic_api.report_error_event.assert_called_once_with(
            project_name=project_name, event=error_pb
        )



# Copyright 2016 Google LLC All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest


class Test_build_flask_context(unittest.TestCase):
    def _call_fut(self, request):
        from google.cloud.error_reporting.util import build_flask_context

        return build_flask_context(request)

    def test_flask_helper(self):
        import mock

        user_agent = mock.Mock(string="Google Cloud Unit Tests Agent")
        request = _Request(
            "http://google.com", "GET", user_agent, "http://gmail.com", "127.0.0.1"
        )

        context = self._call_fut(request)
        self.assertEqual(request.url, context.url)
        self.assertEqual(request.method, context.method)
        self.assertEqual(request.user_agent.string, context.userAgent)
        self.assertEqual(request.referrer, context.referrer)
        self.assertEqual(request.remote_addr, context.remoteIp)


class _Request(object):
    def __init__(self, url, method, user_agent, referrer, remote_addr):
        self.url = url
        self.method = method
        self.user_agent = user_agent
        self.referrer = referrer
        self.remote_addr = remote_addr



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .common import (
    ErrorContext,
    ErrorEvent,
    ErrorGroup,
    HttpRequestContext,
    ServiceContext,
    SourceLocation,
    TrackingIssue,
    ResolutionStatus,
)
from .error_group_service import (
    GetGroupRequest,
    UpdateGroupRequest,
)
from .error_stats_service import (
    DeleteEventsRequest,
    DeleteEventsResponse,
    ErrorGroupStats,
    ListEventsRequest,
    ListEventsResponse,
    ListGroupStatsRequest,
    ListGroupStatsResponse,
    QueryTimeRange,
    ServiceContextFilter,
    TimedCount,
    ErrorGroupOrder,
    TimedCountAlignment,
)
from .report_errors_service import (
    ReportedErrorEvent,
    ReportErrorEventRequest,
    ReportErrorEventResponse,
)

__all__ = (
    "ErrorContext",
    "ErrorEvent",
    "ErrorGroup",
    "HttpRequestContext",
    "ServiceContext",
    "SourceLocation",
    "TrackingIssue",
    "ResolutionStatus",
    "GetGroupRequest",
    "UpdateGroupRequest",
    "DeleteEventsRequest",
    "DeleteEventsResponse",
    "ErrorGroupStats",
    "ListEventsRequest",
    "ListEventsResponse",
    "ListGroupStatsRequest",
    "ListGroupStatsResponse",
    "QueryTimeRange",
    "ServiceContextFilter",
    "TimedCount",
    "ErrorGroupOrder",
    "TimedCountAlignment",
    "ReportedErrorEvent",
    "ReportErrorEventRequest",
    "ReportErrorEventResponse",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.errorreporting_v1beta1.types import common
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.devtools.clouderrorreporting.v1beta1",
    manifest={
        "ReportErrorEventRequest",
        "ReportErrorEventResponse",
        "ReportedErrorEvent",
    },
)


class ReportErrorEventRequest(proto.Message):
    r"""A request for reporting an individual error event.

    Attributes:
        project_name (str):
            Required. The resource name of the Google Cloud Platform
            project. Written as ``projects/{projectId}``, where
            ``{projectId}`` is the `Google Cloud Platform project
            ID <https://support.google.com/cloud/answer/6158840>`__.

            Example: // ``projects/my-project-123``.
        event (google.cloud.errorreporting_v1beta1.types.ReportedErrorEvent):
            Required. The error event to be reported.
    """

    project_name = proto.Field(proto.STRING, number=1,)
    event = proto.Field(proto.MESSAGE, number=2, message="ReportedErrorEvent",)


class ReportErrorEventResponse(proto.Message):
    r"""Response for reporting an individual error event.
    Data may be added to this message in the future.

    """


class ReportedErrorEvent(proto.Message):
    r"""An error event which is reported to the Error Reporting
    system.

    Attributes:
        event_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. Time when the event occurred.
            If not provided, the time when the event was
            received by the Error Reporting system will be
            used.
        service_context (google.cloud.errorreporting_v1beta1.types.ServiceContext):
            Required. The service context in which this
            error has occurred.
        message (str):
            Required. The error message. If no
            ``context.reportLocation`` is provided, the message must
            contain a header (typically consisting of the exception type
            name and an error message) and an exception stack trace in
            one of the supported programming languages and formats.
            Supported languages are Java, Python, JavaScript, Ruby, C#,
            PHP, and Go. Supported stack trace formats are:

            -  **Java**: Must be the return value of
               ```Throwable.printStackTrace()`` <https://docs.oracle.com/javase/7/docs/api/java/lang/Throwable.html#printStackTrace%28%29>`__.
            -  **Python**: Must be the return value of
               ```traceback.format_exc()`` <https://docs.python.org/2/library/traceback.html#traceback.format_exc>`__.
            -  **JavaScript**: Must be the value of
               ```error.stack`` <https://github.com/v8/v8/wiki/Stack-Trace-API>`__
               as returned by V8.
            -  **Ruby**: Must contain frames returned by
               ```Exception.backtrace`` <https://ruby-doc.org/core-2.2.0/Exception.html#method-i-backtrace>`__.
            -  **C#**: Must be the return value of
               ```Exception.ToString()`` <https://msdn.microsoft.com/en-us/library/system.exception.tostring.aspx>`__.
            -  **PHP**: Must start with
               ``PHP (Notice|Parse error|Fatal error|Warning)`` and
               contain the result of
               ```(string)$exception`` <http://php.net/manual/en/exception.tostring.php>`__.
            -  **Go**: Must be the return value of
               ```runtime.Stack()`` <https://golang.org/pkg/runtime/debug/#Stack>`__.
        context (google.cloud.errorreporting_v1beta1.types.ErrorContext):
            Optional. A description of the context in
            which the error occurred.
    """

    event_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
    service_context = proto.Field(
        proto.MESSAGE, number=2, message=common.ServiceContext,
    )
    message = proto.Field(proto.STRING, number=3,)
    context = proto.Field(proto.MESSAGE, number=4, message=common.ErrorContext,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.error_group_service import ErrorGroupServiceClient
from .services.error_group_service import ErrorGroupServiceAsyncClient
from .services.error_stats_service import ErrorStatsServiceClient
from .services.error_stats_service import ErrorStatsServiceAsyncClient
from .services.report_errors_service import ReportErrorsServiceClient
from .services.report_errors_service import ReportErrorsServiceAsyncClient

from .types.common import ErrorContext
from .types.common import ErrorEvent
from .types.common import ErrorGroup
from .types.common import HttpRequestContext
from .types.common import ServiceContext
from .types.common import SourceLocation
from .types.common import TrackingIssue
from .types.common import ResolutionStatus
from .types.error_group_service import GetGroupRequest
from .types.error_group_service import UpdateGroupRequest
from .types.error_stats_service import DeleteEventsRequest
from .types.error_stats_service import DeleteEventsResponse
from .types.error_stats_service import ErrorGroupStats
from .types.error_stats_service import ListEventsRequest
from .types.error_stats_service import ListEventsResponse
from .types.error_stats_service import ListGroupStatsRequest
from .types.error_stats_service import ListGroupStatsResponse
from .types.error_stats_service import QueryTimeRange
from .types.error_stats_service import ServiceContextFilter
from .types.error_stats_service import TimedCount
from .types.error_stats_service import ErrorGroupOrder
from .types.error_stats_service import TimedCountAlignment
from .types.report_errors_service import ReportedErrorEvent
from .types.report_errors_service import ReportErrorEventRequest
from .types.report_errors_service import ReportErrorEventResponse

__all__ = (
    "ErrorGroupServiceAsyncClient",
    "ErrorStatsServiceAsyncClient",
    "ReportErrorsServiceAsyncClient",
    "DeleteEventsRequest",
    "DeleteEventsResponse",
    "ErrorContext",
    "ErrorEvent",
    "ErrorGroup",
    "ErrorGroupOrder",
    "ErrorGroupServiceClient",
    "ErrorGroupStats",
    "ErrorStatsServiceClient",
    "GetGroupRequest",
    "HttpRequestContext",
    "ListEventsRequest",
    "ListEventsResponse",
    "ListGroupStatsRequest",
    "ListGroupStatsResponse",
    "QueryTimeRange",
    "ReportErrorEventRequest",
    "ReportErrorEventResponse",
    "ReportErrorsServiceClient",
    "ReportedErrorEvent",
    "ResolutionStatus",
    "ServiceContext",
    "ServiceContextFilter",
    "SourceLocation",
    "TimedCount",
    "TimedCountAlignment",
    "TrackingIssue",
    "UpdateGroupRequest",
)



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest

import mock


def _make_credentials():
    import google.auth.credentials

    return mock.Mock(spec=google.auth.credentials.Credentials)


class TestClient(unittest.TestCase):

    PROJECT = "PROJECT"
    SERVICE = "SERVICE"
    VERSION = "myversion"

    @staticmethod
    def _get_target_class():
        from google.cloud.error_reporting.client import Client

        return Client

    def _make_one(self, *args, **kw):
        return self._get_target_class()(*args, **kw)

    def _make_http(self, *args, **kw):
        from google.cloud.error_reporting.client import HTTPContext

        return HTTPContext(*args, **kw)

    def _get_report_payload(self, error_api):
        self.assertEqual(error_api.report_error_event.call_count, 1)
        call = error_api.report_error_event.mock_calls[0]
        _, positional, kwargs = call
        self.assertEqual(kwargs, {})
        self.assertEqual(len(positional), 1)
        return positional[0]

    @mock.patch("google.cloud.client._determine_default_project")
    def test_ctor_defaults(self, default_mock):
        from google.api_core.client_info import ClientInfo

        credentials = _make_credentials()
        default_mock.return_value = "foo"
        client = self._make_one(credentials=credentials)
        self.assertEqual(client.service, client.DEFAULT_SERVICE)
        self.assertEqual(client.version, None)
        self.assertIsInstance(client._client_info, ClientInfo)
        default_mock.assert_called_once_with(None)

    def test_ctor_explicit(self):
        credentials = _make_credentials()
        client_info = mock.Mock()
        client_options = mock.Mock()
        client = self._make_one(
            project=self.PROJECT,
            credentials=credentials,
            service=self.SERVICE,
            version=self.VERSION,
            client_info=client_info,
            client_options=client_options,
        )
        self.assertEqual(client.service, self.SERVICE)
        self.assertEqual(client.version, self.VERSION)
        self.assertIs(client._client_info, client_info)
        self.assertIs(client._client_options, client_options)

    def test_report_errors_api_already(self):
        credentials = _make_credentials()
        client = self._make_one(project=self.PROJECT, credentials=credentials)
        client._report_errors_api = already = mock.Mock()
        self.assertIs(client.report_errors_api, already)

    def test_report_errors_api_wo_grpc(self):
        credentials = _make_credentials()
        client_info = mock.Mock()
        client_options = mock.Mock()
        http = mock.Mock()
        client = self._make_one(
            project=self.PROJECT,
            credentials=credentials,
            client_info=client_info,
            client_options=client_options,
            _http=http,
            _use_grpc=False,
        )
        patch = mock.patch(
            "google.cloud.error_reporting.client._ErrorReportingLoggingAPI"
        )

        with patch as patched:
            api = client.report_errors_api

        self.assertIs(api, patched.return_value)
        patched.assert_called_once_with(
            self.PROJECT, credentials, http, client_info, client_options
        )

    def test_report_errors_api_w_grpc(self):
        credentials = _make_credentials()
        client = self._make_one(
            project=self.PROJECT, credentials=credentials, _use_grpc=True
        )
        patch = mock.patch("google.cloud.error_reporting.client.make_report_error_api")

        with patch as patched:
            api = client.report_errors_api

        self.assertIs(api, patched.return_value)
        patched.assert_called_once_with(client)

    def test_report_exception_with_grpc(self):
        credentials = _make_credentials()
        client = self._make_one(project=self.PROJECT, credentials=credentials)

        patch = mock.patch("google.cloud.error_reporting.client.make_report_error_api")
        with patch as make_api:
            try:
                raise NameError
            except NameError:
                client.report_exception()
            payload = make_api.return_value.report_error_event.call_args[0][0]
            make_api.assert_called_once_with(client)

        self.assertEqual(payload["serviceContext"], {"service": client.DEFAULT_SERVICE})
        self.assertIn("test_report", payload["message"])
        self.assertIn("test_client.py", payload["message"])

    def test_report_exception_wo_grpc(self):
        credentials = _make_credentials()
        client = self._make_one(
            project=self.PROJECT, credentials=credentials, _use_grpc=False
        )
        patch = mock.patch(
            "google.cloud.error_reporting.client._ErrorReportingLoggingAPI"
        )
        with patch as _error_api:
            try:
                raise NameError
            except NameError:
                client.report_exception()
            mock_report = _error_api.return_value.report_error_event
            payload = mock_report.call_args[0][0]

        self.assertEqual(payload["serviceContext"], {"service": client.DEFAULT_SERVICE})
        self.assertIn("test_report", payload["message"])
        self.assertIn("test_client.py", payload["message"])
        self.assertIsNotNone(client.report_errors_api)

    @mock.patch("google.cloud.error_reporting.client.make_report_error_api")
    def test_report_exception_with_service_version_in_constructor(self, make_api):
        credentials = _make_credentials()
        service = "notdefault"
        version = "notdefaultversion"
        client = self._make_one(
            project=self.PROJECT,
            credentials=credentials,
            service=service,
            version=version,
        )

        http_context = self._make_http(method="GET", response_status_code=500)
        user = "user@gmail.com"

        error_api = mock.Mock(spec=["report_error_event"])
        make_api.return_value = error_api

        try:
            raise NameError
        except NameError:
            client.report_exception(http_context=http_context, user=user)

        make_api.assert_called_once_with(client)

        payload = self._get_report_payload(error_api)
        self.assertEqual(
            payload["serviceContext"], {"service": service, "version": version}
        )
        self.assertIn(
            "test_report_exception_with_service_version_in_constructor",
            payload["message"],
        )
        self.assertIn("test_client.py", payload["message"])
        self.assertEqual(payload["context"]["httpRequest"]["responseStatusCode"], 500)
        self.assertEqual(payload["context"]["httpRequest"]["method"], "GET")
        self.assertEqual(payload["context"]["user"], user)

    @mock.patch("google.cloud.error_reporting.client.make_report_error_api")
    def test_report(self, make_api):
        credentials = _make_credentials()
        client = self._make_one(project=self.PROJECT, credentials=credentials)

        error_api = mock.Mock(spec=["report_error_event"])
        make_api.return_value = error_api

        message = "this is an error"
        client.report(message)

        payload = self._get_report_payload(error_api)

        self.assertEqual(payload["message"], message)
        report_location = payload["context"]["reportLocation"]
        self.assertIn("test_client.py", report_location["filePath"])
        self.assertEqual(report_location["functionName"], "test_report")
        self.assertGreater(report_location["lineNumber"], 100)
        self.assertLess(report_location["lineNumber"], 250)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.devtools.clouderrorreporting.v1beta1",
    manifest={
        "ResolutionStatus",
        "ErrorGroup",
        "TrackingIssue",
        "ErrorEvent",
        "ServiceContext",
        "ErrorContext",
        "HttpRequestContext",
        "SourceLocation",
    },
)


class ResolutionStatus(proto.Enum):
    r"""Resolution status of an error group."""
    RESOLUTION_STATUS_UNSPECIFIED = 0
    OPEN = 1
    ACKNOWLEDGED = 2
    RESOLVED = 3
    MUTED = 4


class ErrorGroup(proto.Message):
    r"""Description of a group of similar error events.

    Attributes:
        name (str):
            The group resource name.
            Example:
            <code>projects/my-project-123/groups/CNSgkpnppqKCUw</code>
        group_id (str):
            Group IDs are unique for a given project. If
            the same kind of error occurs in different
            service contexts, it will receive the same group
            ID.
        tracking_issues (Sequence[google.cloud.errorreporting_v1beta1.types.TrackingIssue]):
            Associated tracking issues.
        resolution_status (google.cloud.errorreporting_v1beta1.types.ResolutionStatus):
            Error group's resolution status.
            An unspecified resolution status will be
            interpreted as OPEN
    """

    name = proto.Field(proto.STRING, number=1,)
    group_id = proto.Field(proto.STRING, number=2,)
    tracking_issues = proto.RepeatedField(
        proto.MESSAGE, number=3, message="TrackingIssue",
    )
    resolution_status = proto.Field(proto.ENUM, number=5, enum="ResolutionStatus",)


class TrackingIssue(proto.Message):
    r"""Information related to tracking the progress on resolving the
    error.

    Attributes:
        url (str):
            A URL pointing to a related entry in an issue tracking
            system. Example:
            ``https://github.com/user/project/issues/4``
    """

    url = proto.Field(proto.STRING, number=1,)


class ErrorEvent(proto.Message):
    r"""An error event which is returned by the Error Reporting
    system.

    Attributes:
        event_time (google.protobuf.timestamp_pb2.Timestamp):
            Time when the event occurred as provided in
            the error report. If the report did not contain
            a timestamp, the time the error was received by
            the Error Reporting system is used.
        service_context (google.cloud.errorreporting_v1beta1.types.ServiceContext):
            The ``ServiceContext`` for which this error was reported.
        message (str):
            The stack trace that was reported or logged
            by the service.
        context (google.cloud.errorreporting_v1beta1.types.ErrorContext):
            Data about the context in which the error
            occurred.
    """

    event_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
    service_context = proto.Field(proto.MESSAGE, number=2, message="ServiceContext",)
    message = proto.Field(proto.STRING, number=3,)
    context = proto.Field(proto.MESSAGE, number=5, message="ErrorContext",)


class ServiceContext(proto.Message):
    r"""Describes a running service that sends errors.
    Its version changes over time and multiple versions can run in
    parallel.

    Attributes:
        service (str):
            An identifier of the service, such as the name of the
            executable, job, or Google App Engine service name. This
            field is expected to have a low number of values that are
            relatively stable over time, as opposed to ``version``,
            which can be changed whenever new code is deployed.

            Contains the service name for error reports extracted from
            Google App Engine logs or ``default`` if the App Engine
            default service is used.
        version (str):
            Represents the source code version that the
            developer provided, which could represent a
            version label or a Git SHA-1 hash, for example.
            For App Engine standard environment, the version
            is set to the version of the app.
        resource_type (str):
            Type of the MonitoredResource. List of
            possible values:
            https://cloud.google.com/monitoring/api/resources
            Value is set automatically for incoming errors
            and must not be set when reporting errors.
    """

    service = proto.Field(proto.STRING, number=2,)
    version = proto.Field(proto.STRING, number=3,)
    resource_type = proto.Field(proto.STRING, number=4,)


class ErrorContext(proto.Message):
    r"""A description of the context in which an error occurred.
    This data should be provided by the application when reporting
    an error, unless the
    error report has been generated automatically from Google App
    Engine logs.

    Attributes:
        http_request (google.cloud.errorreporting_v1beta1.types.HttpRequestContext):
            The HTTP request which was processed when the
            error was triggered.
        user (str):
            The user who caused or was affected by the crash. This can
            be a user ID, an email address, or an arbitrary token that
            uniquely identifies the user. When sending an error report,
            leave this field empty if the user was not logged in. In
            this case the Error Reporting system will use other data,
            such as remote IP address, to distinguish affected users.
            See ``affected_users_count`` in ``ErrorGroupStats``.
        report_location (google.cloud.errorreporting_v1beta1.types.SourceLocation):
            The location in the source code where the
            decision was made to report the error, usually
            the place where it was logged. For a logged
            exception this would be the source line where
            the exception is logged, usually close to the
            place where it was caught.
    """

    http_request = proto.Field(proto.MESSAGE, number=1, message="HttpRequestContext",)
    user = proto.Field(proto.STRING, number=2,)
    report_location = proto.Field(proto.MESSAGE, number=3, message="SourceLocation",)


class HttpRequestContext(proto.Message):
    r"""HTTP request data that is related to a reported error.
    This data should be provided by the application when reporting
    an error, unless the
    error report has been generated automatically from Google App
    Engine logs.

    Attributes:
        method (str):
            The type of HTTP request, such as ``GET``, ``POST``, etc.
        url (str):
            The URL of the request.
        user_agent (str):
            The user agent information that is provided
            with the request.
        referrer (str):
            The referrer information that is provided
            with the request.
        response_status_code (int):
            The HTTP response status code for the
            request.
        remote_ip (str):
            The IP address from which the request
            originated. This can be IPv4, IPv6, or a token
            which is derived from the IP address, depending
            on the data that has been provided in the error
            report.
    """

    method = proto.Field(proto.STRING, number=1,)
    url = proto.Field(proto.STRING, number=2,)
    user_agent = proto.Field(proto.STRING, number=3,)
    referrer = proto.Field(proto.STRING, number=4,)
    response_status_code = proto.Field(proto.INT32, number=5,)
    remote_ip = proto.Field(proto.STRING, number=6,)


class SourceLocation(proto.Message):
    r"""Indicates a location in the source code of the service for which
    errors are reported. ``functionName`` must be provided by the
    application when reporting an error, unless the error report
    contains a ``message`` with a supported exception stack trace. All
    fields are optional for the later case.

    Attributes:
        file_path (str):
            The source code filename, which can include a
            truncated relative path, or a full path from a
            production machine.
        line_number (int):
            1-based. 0 indicates that the line number is
            unknown.
        function_name (str):
            Human-readable name of a function or method. The value can
            include optional context like the class or package name. For
            example, ``my.package.MyClass.method`` in case of Java.
    """

    file_path = proto.Field(proto.STRING, number=1,)
    line_number = proto.Field(proto.INT32, number=2,)
    function_name = proto.Field(proto.STRING, number=4,)


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2016 Google LLC All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""GAX wrapper for Error Reporting API requests."""
import json

import google.cloud.errorreporting_v1beta1


def make_report_error_api(client):
    """Create an instance of the gapic Logging API.

    :type client::class:`google.cloud.error_reporting.Client`
    :param client: Error Reporting client.

    :rtype: :class:_ErrorReportingGapicApi
    :returns: An Error Reporting API instance.
    """
    gapic_api = google.cloud.errorreporting_v1beta1.ReportErrorsServiceClient(
        credentials=client._credentials,
        client_info=client._client_info,
        client_options=client._client_options,
    )
    return _ErrorReportingGapicApi(gapic_api, client.project)


class _ErrorReportingGapicApi(object):
    """Helper mapping Error Reporting-related APIs

    :type gapic:
        :class:`google.cloud.errorreporting_v1beta1.ReportErrorsServiceClient`
    :param gapic: API object used to make RPCs.

    :type project: str
    :param project: Google Cloud Project ID
    """

    def __init__(self, gapic_api, project):
        self._gapic_api = gapic_api
        self._project = project

    def report_error_event(self, error_report):
        """Uses the gapic client to report the error.

        :type error_report: dict
        :param error_report:
            payload of the error report formatted according to
            https://cloud.google.com/error-reporting/docs/formatting-error-messages
            This object should be built using
            Use
            :meth:~`google.cloud.error_reporting.client._build_error_report`
        """
        project_name = f"projects/{self._project}"

        # Since error_report uses camel case for key names (like serviceContext),
        # but ReportedErrorEvent uses snake case for key names (like service_context),
        # we need to route throught json.
        error_report_payload = google.cloud.errorreporting_v1beta1.ReportedErrorEvent.from_json(
            json.dumps(error_report)
        )

        self._gapic_api.report_error_event(
            project_name=project_name, event=error_report_payload
        )



# Copyright 2017 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import operator
import unittest
import mock

from google.cloud import error_reporting
import google.cloud.errorreporting_v1beta1
from google.protobuf.duration_pb2 import Duration

from test_utils.retry import RetryResult
from test_utils.system import unique_resource_id


ERROR_MSG = "Error Reporting System Test"


def setUpModule():
    Config.CLIENT = error_reporting.Client()


class Config(object):
    """Run-time configuration to be modified at set-up.

    This is a mutable stand-in to allow test set-up to modify
    global state.
    """

    CLIENT = None


def _list_groups(client):
    """List Error Groups from the last 60 seconds.

    This class provides a wrapper around making calls to the GAX
    API. It's used by the system tests to find the appropriate error group
    to verify the error was successfully reported.

    :type client: :class:`~google.cloud.error_reporting.client.Client`
    :param client: The client containing a project and credentials.

    :rtype: :class:`~google.gax.ResourceIterator`
    :returns: Iterable of :class:`~.google.cloud.errorreporting_v1beta1.ErrorGroupStats`.
    """
    gax_api = google.cloud.errorreporting_v1beta1.ErrorStatsServiceClient(
        credentials=client._credentials
    )
    project_name = f"projects/{client.project}"

    time_range = google.cloud.errorreporting_v1beta1.QueryTimeRange()
    time_range.period = (
        google.cloud.errorreporting_v1beta1.QueryTimeRange.Period.PERIOD_1_HOUR
    )

    duration = Duration(seconds=60 * 60)

    return gax_api.list_group_stats(
        request={
            "project_name": project_name,
            "time_range": time_range,
            "timed_count_duration": duration,
        }
    )


def _simulate_exception(class_name, client):
    """Simulates an exception to verify it was reported.

    :type class_name: str
    :param class_name: The name of a custom error class to
                       create (and raise).

    :type client: :class:`~google.cloud.error_reporting.client.Client`
    :param client: The client that will report the exception.
    """
    custom_exc = type(class_name, (RuntimeError,), {})
    try:
        raise custom_exc(ERROR_MSG)
    except RuntimeError:
        client.report_exception()


def _get_error_count(class_name, client):
    """Counts the number of errors in the group of the test exception.

    :type class_name: str
    :param class_name: The name of a custom error class used.

    :type client: :class:`~google.cloud.error_reporting.client.Client`
    :param client: The client containing a project and credentials.

    :rtype: int
    :returns: Group count for errors that match ``class_name``. If no
              match is found, returns :data:`None`.
    """
    groups = _list_groups(client)
    for group in groups:
        if class_name in group.representative.message:
            return group.count


class TestErrorReporting(unittest.TestCase):
    def test_report_exception(self):
        # Get a class name unique to this test case.
        class_name = "RuntimeError" + unique_resource_id("_")

        # Simulate an error: group won't exist until we report
        # first exception.
        _simulate_exception(class_name, Config.CLIENT)

        is_one = functools.partial(operator.eq, 1)
        is_one.__name__ = "is_one"  # partial() has no name.
        retry = RetryResult(is_one, max_tries=8)
        wrapped_get_count = retry(_get_error_count)

        error_count = wrapped_get_count(class_name, Config.CLIENT)
        self.assertEqual(error_count, 1)

    def test_report_exception_no_grpc(self):
        with mock.patch.dict(
            "os.environ", {"GOOGLE_CLOUD_DISABLE_GRPC": "true"}, clear=True
        ):
            self.test_report_exception()
